<?php

namespace App\Services\goods;

use Api\PhpUtils\Common\BaseConvert;
use Api\PhpUtils\Elastic\ElasticUtil;
use App\Models\goods\mysql\GoodsSku;

class ElasticGoodService
{
    const INDEX = "bp_goods";

    public static function createIndex()
    {
        $elasticClient = ElasticUtil::getInstance();
        $data = [
            "settings" => [
                "analysis" => [
                    "analyzer" => [
                        "default" => [
                            "type" => "ik_max_word",
                        ]
                    ]
                ]
            ],
            'mappings' => [
                'properties' => [
                    'goods_name' => [
                        'type' => 'text',
                        'analyzer' => 'ik_max_word',
                    ],
                    'desc' => [
                        'type' => 'text',
                        'analyzer' => 'ik_max_word',
                    ],
                    'category_1_name' => [
                        'type' => 'text',
                        'analyzer' => 'ik_max_word',
                    ],
                    'category_2_name' => [
                        'type' => 'text',
                        'analyzer' => 'ik_max_word',
                    ],
                    'life_account_name' => [
                        'type' => 'text',
                        'analyzer' => 'ik_max_word',
                    ],
                    'shop_name' => [
                        'type' => 'text',
                        'analyzer' => 'ik_max_word',
                    ],
                    'shop_location' => [
                        'type' => 'geo_point',
                    ],
                ]
            ]
        ];
        return $elasticClient->createIndex(self::INDEX, $data);
    }

    /**
     * 删除
     */
    public static function deleteIndex(): void
    {
        $elasticClient = ElasticUtil::getInstance();

        $res = $elasticClient->deleteIndex(self::INDEX);
        var_dump($res);
    }

    /**
     * 写入 es 一条数据
     * @param $data
     * @param $id
     * @return array|callable
     */
    public static function indexDoc($id, $data)
    {
        $elasticClient = ElasticUtil::getInstance();
        return $elasticClient->indexDoc(self::INDEX, $data, $id);
    }

    /**
     * 修改 es 一条数据
     * @param $id
     * @param $data
     * @return array|callable
     */
    public static function updateDoc($id, $data)
    {
        $elasticClient = ElasticUtil::getInstance();
        return $elasticClient->updateDoc(self::INDEX, $id, $data);
    }

    public static function bulk($index)
    {
        $elasticClient = ElasticUtil::getInstance();

        $data = [
            ['index' => ['_index' => $index]],
            ['test_field' => '公安部：各地校车将享最高路权'],
            ['index' => ['_index' => $index]],
            ['test_field' => '中韩渔警冲突调查：韩警平均每天扣1艘中国渔船'],
            ['index' => ['_index' => $index]],
            ['test_field' => '中国驻洛杉矶领事馆遭亚裔男子枪击 嫌犯已自首'],
        ];
        $res = $elasticClient->bulkIndexDoc($data);
        var_export($res);
    }

    /**
     * @param array $params
     * @return array|object
     */
    public static function searchDoc($params = [])
    {
        $sortType = 0;
        if (!empty($params["sort_type"])) {
            $sortType = $params["sort_type"];
        }
        $from = empty($params["from"]) ? 0 : (int)$params["from"];
        $size = empty($params["size"]) ? 10 : (int)$params["size"];

        $sortList = [
            ["price_sort" => "asc", "total_amount_sold" => "desc"],
            ["total_amount_sold" => "desc"],
            ["price_sort" => "asc"],
        ];
        $data = [
            "from" => $from,
            "size" => $size,
            "sort" => $sortList[$sortType],
        ];

        if (!empty($params["search_content"])) {
            $data["query"]["bool"]["must"]["multi_match"] = [
                "query" => $params["search_content"],
                "fields" => ["goods_name", "category_1_name", "category_2_name", "desc", "life_account_name", "shop_name"],
            ];
        }

        // 注意这里顺序，必须 multi_match 在前，term在后
        $data["query"]["bool"]["must"] = [
            ["term" => ["audit_status" => GoodsSku::STATUS_PASS]],
            ["term" => ["online_status" => GoodsSku::ONLINE_STATUS_ONLINE]],
        ];

        if (!empty($params["latitude"]) && !empty($params["longitude"])) {
            $data["query"]["bool"]["filter"]["geo_distance"] = [
                "distance" => "50km",
                "shop_location" => [
                    "lat" => $params["latitude"],
                    "lon" => $params["longitude"],
                ]
            ];

            $data["sort"]["_geo_distance"] = [
                "shop_location" => [
                    "lat" => $params["latitude"],
                    "lon" => $params["longitude"],
                ],
                "order" => "asc",
                "unit" => "km",
                "distance_type" => "plane",
            ];
        }

        $elasticClient = ElasticUtil::getInstance();
        $res = $elasticClient->searchDoc(self::INDEX, $data);

        $result = [];
        if (!empty($res["hits"]["total"]["value"])) {
            $result["count"] = $res["hits"]["total"]["value"];
            foreach ($res["hits"]["hits"] as $key => $source) {


                $result["list"][$key]["goods_desc_pic_url_list"] = [];
                if (!empty($source["_source"]["goods_desc_pic_url"])) {
                    $ksyun = GoodsService::getUrlList($source["_source"]["goods_desc_pic_url"]);
                    $result["list"][$key]["goods_desc_pic_url_list"] = $ksyun;
                }

                $result["list"][$key]["goods_sku_id"] = $source["_source"]["goods_sku_id"];
                $result["list"][$key]["goods_desc_pic_url"] = $source["_source"]["goods_desc_pic_url"];
                $result["list"][$key]["goods_name"] = $source["_source"]["goods_name"];
                $result["list"][$key]["inventory_rest"] = $source["_source"]["inventory_rest"];
                $result["list"][$key]["total_amount_sold"] = $source["_source"]["total_amount_sold"];
                $result["list"][$key]["original_price"] = $source["_source"]["original_price"];
                $result["list"][$key]["price"] = $source["_source"]["price"];
                $result["list"][$key]["audit_status"] = $source["_source"]["audit_status"];
                $result["list"][$key]["online_status"] = $source["_source"]["online_status"];
                $result["list"][$key]["shop_name"] = $source["_source"]["shop_name"];
                $result["list"][$key]["shop_location"] = $source["_source"]["shop_location"];
                $result["list"][$key]["life_account_name"] = $source["_source"]["life_account_name"];
                $result["list"][$key]["life_account_icon"] = $source["_source"]["life_account_icon"];
                $result["list"][$key]["distance"] = empty($source["sort"][2]) ? "" : round($source["sort"][2], 2) . "km";

            }
        }
        //为避免空数据时候返回给前端错误结构
        return $result ? $result : (object)$result;
    }

    /**
     * id 在es数据里是否存在
     * @param $skuId
     * @return bool
     */
    public static function isHaveById($skuId)
    {
        $data["query"]["bool"]["must"] = [
            ["term" => ["_id" => $skuId]],
        ];

        $elasticClient = ElasticUtil::getInstance();
        $res = $elasticClient->searchDoc(self::INDEX, $data);

        if (!empty($res["hits"]["total"]["value"])) {
            return $res["hits"]["total"]["value"] > 0;
        }
        return false;
    }
}